@@ -28,15 +28,17 @@ def createParser():
 def check_socket(host, port):
     with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
         if sock.connect_ex((host, port)) == 0:
-            #print('{host} {port} exist'.format(host=host, port=port))
-            sys.exit(0)
+            print('{host} {port} exist'.format(host=host, port=port))
+            #sys.exit(0)
         else:
-            #print('{host} {port} unexist'.format(host=host, port=port))
-            sys.exit(1)
+            print('{host} {port} none'.format(host=host, port=port))
+            #sys.exit(1)
             
 
 if __name__ == "__main__":
     parser = createParser()
     namespace = parser.parse_args() 
+    #print(namespace.host, namespace.port)
     kwargs={'host':namespace.host, "port":int(namespace.port)}
-    check_socket(**kwargs)
\ No newline at end of file
+    check_socket(**kwargs)
+    
