@@ -16,8 +16,9 @@ from srv.dockerfiles import DockerfileFactory
 
 from fabric.tasks import Task, WrappedCallableTask, get_task_details
 
-from fabric2 import SerialGroup as Group
+#from fabric2 import SerialGroup as Group
 from fabric2 import Connection
+from _operator import add
 
 def fname(function_to_decorate):
     def the_wrapper_around_the_original_function():
@@ -105,6 +106,10 @@ class Element(ABC):
         for obj in vars(self).values():
             if isinstance(obj, WrappedCallableTask):
                 obj.roles=role
+    def updateOpt(self):
+        publish=(['2222:2223'])
+        setattr(self._service.options, 'publich', publish)
+        pass
     
     #@abstractmethod     
     def initRepoForLayer(self, branch='master'):
@@ -113,12 +118,12 @@ class Element(ABC):
             rep=self._gitrepos.getRepo(workdir=self.layerPath, cur_branch=branch)
         except Exception as ex:
             print(ex)
-        return rep
+            return rep
     
     def convertToService(self, tag='v1'):
         service=docker.Service(
-            name=self.imageTag,
-            image=self.serviceName+':'+tag,
+            name=self.serviceName,
+            image=self.imageTag+':'+tag,
             options=self.opt,
             )
         self.setService(service)
@@ -143,7 +148,7 @@ class Element(ABC):
     @fab.task
     def commit_task(self, tag='v1'):
         cmd='docker commit {ID} {newServiceTag}'.format(ID=self.service.info['Id'], 
-                                                        newServiceTag=self.registry+'/'+self.serviceName+':'+tag)
+                                                        newServiceTag=self.registry+'/'+self.imageTag+':'+tag)
         fab.run(cmd)
         self.push(tag=tag)
         fab.env.host=None
@@ -190,7 +195,7 @@ class NetworkManager(Manager):
 class RoleManager(list):
     def __new__(cls, role):
         cls.role=role
-        cls.portIndex=23750
+        cls.portIndex=2375
         return super().__new__(cls)
     #def __init__(self, role, addr):
         #self.role=role
@@ -200,39 +205,56 @@ class RoleManager(list):
     
     def executeRemoteModules(self, moduleName, *args, **kwargs):
         fullname=os.path.abspath(moduleName.__file__)
-        self.put(source=fullname, *args, **kwargs)
+        name=os.path.basename(fullname)
+        runString='python3 /tmp/{name} '.format(name=name)
+        for arg,val in kwargs.items():
+            runString=runString+'--'+str(arg)+' '+str(val)+' '
+        self.put(fullname, remote='/tmp/{name}'.format(name=name), preserve_mode=False)
+        result=self.run(runString, hide='both')
+        return result
         
     
     def execute(self, callFunc):
         
         import paramiko
         from docker import DockerClient
-        from sshtunnel import SSHTunnelForwarder
+        #from sshtunnel import SSHTunnelForwarder
+        import sshtunnel
         import time
 
         for host in self.adress:
             user=host.split('@', 1)[0]
             addr=host.split('@', 1)[1]
-            port=self.localConnectionPort
-            forward=SSHTunnelForwarder((addr, 22), 
-                                ssh_username=user, 
-                                ssh_pkey="/var/ssh/rsa_key", 
-                                ssh_private_key_password="secret", 
-                                remote_bind_address=(addr, 2375), 
-                                local_bind_address=('127.0.0.1', port))
-            forward.start()
-            time.sleep(3)
-            dockerConection=DockerClient('tcp://127.0.0.1:{port}'.format(port=port))
-            res=callFunc(dockerConection)
-            dockerConection.close()
-            del dockerConection
-            forward.stop()
+            port=self.chekFreePort(self.portIndex)
+            with sshtunnel.open_tunnel(
+                (addr, 22),
+                ssh_username=user,
+                ssh_pkey=self.pkey,
+                #ssh_private_key_password="secret",
+                remote_bind_address=(addr, 2375),
+                local_bind_address=('127.0.0.1', port)
+            ) as tunnel:
+                
+            #forward=SSHTunnelForwarder((addr, 22), 
+            #                    ssh_username=user, 
+            #                    #ssh_pkey="~/.ssh/id_rsa", 
+            #                    #ssh_private_key_password="secret", 
+            #                    remote_bind_address=(addr, 2375), 
+            #                    local_bind_address=('127.0.0.1', port))
+            #forward.start()
+                #time.sleep(3)
+                dockerConection=DockerClient('tcp://127.0.0.1:{port}'.format(port=port))
+                res=callFunc(dockerConection)
+                dockerConection.close()
+                del dockerConection
+            #forward.stop()
         return res
     
-    @property
-    def localConnectionPort(self):
-        port=self.portIndex
-        self.portIndex+=1
+
+    def chekFreePort(self, port='2375'):
+        from fabula.remoteModules import chkPort
+        while 'exist' in self.executeRemoteModules(chkPort, host='127.0.0.1', port=port).stdout:
+            port+=1 
         return port
     
     def getConnections(self):
@@ -246,7 +268,14 @@ class RoleManager(list):
     @adress.setter
     def adress(self, addr):
         self.addr=addr
-        
+    
+    @property
+    def pkey(self):
+        return self.key
+    @pkey.setter
+    def pkey(self, key):
+        self.key=key 
+       
     @property
     def roles(self):
         return self.role
@@ -257,39 +286,50 @@ class RoleManager(list):
     #@role.deleter
     #def roles(self, rolename):
     #    del self.role_list.remove(rolename)
-    def run(self, arg):
+    def run(self, command, **kwargs):
         for host in self.adress:
-            result = Connection(host).run(arg, pty=True)
-            print("{}: {}".format(host, result.stdout.strip()))
+            result=0
+            try:
+                result = Connection(host).run(command, **kwargs)
+                return result
+            except Exception as Ex:
+                return Ex
+            #result = Connection(host).run('ls /tmp', hide='both')
+            #print("{}: {}".format(host, result.stdout.strip()))
     
     def put(self, *args, **kwargs):
-        for host in self.adress:
-            result = Connection(host).put(kwargs['source'], remote='/tmp/remmod.py', preserve_mode=False)
-            result = Connection(host).run('ls /tmp', hide='both')
-            print("{}: {}".format(host, result.stdout.strip()))
-            print(host)
-    
+        for host in self.adress:         
+            #result = Connection(host).put(source, remote='/tmp/{name}'.format(name=name), preserve_mode=False)
+            result = Connection(host).put(*args, **kwargs)
+            #print("{}: {}".format(host, result.stdout.strip()))
+            return result
     
     
 class VolumeManager(Manager):
-    def __init__(self, role, typeVol='volume', driver='local', source='defvol', options='readonly=false'):
-        self.role=role
+    def __init__(self, typeVol='volume', driver='local', source='defvol', options='readonly=false'):
+        #self.role=role
         self.driver=driver
         self.source=source
         self.typeVol=typeVol
+        
         self.options=options
         self.template='type={type},source={source},destination={destination},{options}'
     def setDestination(self, destination):
         self.destination=destination
     def setSource(self, source):
         self.source=source
+    def setRole(self, role):
+        self.role=role
     def setType(self, typeVol):
         self.typeVol=typeVol
     def setAditionalOpt(self, options):
         self.options=options
+    def setDriverOpt(self, **kwargs):
+        self.driver_opts=kwargs
     
-    def update(self):
-        res=self.role.execute(self._create)
+    def update(self, role):
+        #'получить порт и адрес сторожки'
+        res=role.execute(self._create)
         return res
     
     def _create(self, connection):
@@ -297,7 +337,7 @@ class VolumeManager(Manager):
             vol=connection.volumes.get(self.source)
             print("Volume {name} exist".format(name=vol))
         except Exception as Ex:
-            res=connection.volumes.create(self.source, driver=self.driver)
+            res=connection.volumes.create(self.source, driver=self.driver, driver_opts=self.driver_opts)
             res
         return False
     
@@ -311,8 +351,10 @@ class BlackBox(ABC):
     '''
     classdocs
     '''
-    stack={}
-    __index=0
+    eStack={}
+    sStack={}
+    __eIndex=0
+    __sIndex=0
     
     projectName=None
     
@@ -326,30 +368,44 @@ class BlackBox(ABC):
     #    pass
     
     @property
-    def layer(self):
-        return self.list
+    def elements(self):
+        return self.eStack
     
-    @layer.setter
-    def layer(self, strategy):
+    @elements.setter
+    def elements(self, element):
         
-        self.stack[strategy._element.imageTag]=strategy
-        self._increment()
+        self.eStack[element.imageTag]=element
+        #self._eIncrement()
+    
+    @property
+    def strategies(self):
+        return self.sStack
+    
+    @strategies.setter
+    def strategies(self, strategy):
         
-    def _increment(self):
-        self.__index+=10
+        self.sStack[strategy._element.imageTag]=strategy
+        self._sIncrement()
+    
+    def _sIncrement(self):
+        self.__sIndex+=10
+    
+    def _eIncrement(self):
+        self.__eIndex+=10
     
     @fab.hosts()
     @fab.roles()
     @fab.task    
     def compile_layers(self):
-        for item in self.stack.keys():
-            self.stack[item].fullcycle()
+        for item in self.strategies.keys():
+            self.strategies[item].fullcycle()
+            
     @fab.hosts()
     @fab.roles()
     @fab.task    
     def initAllRepo(self):
-        for item in self.stack.keys():
-            self.stack[item]._element.runrepo()
+        for item in self.strategies.keys():
+            self.strategies[item]._element.runrepo()
     
     @fab.hosts()
     @fab.roles()
@@ -361,10 +417,10 @@ class BlackBox(ABC):
 
     
 
-class LayerBuilder(ImageBuildDockerTasks, Tasks, Element):
+class LayerBuilder(ImageBuildDockerTasks, Element):
     
     def __init__(self,
-                net='network',
+                net='defnetwork',
                 role=['defrole'], 
                 dockerlogin=False, registry='',
                 projectname='testproj', 
@@ -375,17 +431,19 @@ class LayerBuilder(ImageBuildDockerTasks, Tasks, Element):
         self.myname=projectname
         self.registry=registry
         self.imageTag=imageTag
-        self.serviceName=projectname+'/'+imageTag
+        self.serviceName=projectname+'-'+os.path.basename(imageTag)
+        self.servicePath=projectname+'/'+imageTag
         self.serviceTag=projectname+'/'+imageTag+':'+release
-        self.layerPath=os.path.join(os.getcwd(), 'build/'+self.serviceName)
+        self.layerPath=os.path.join(os.getcwd(), 'build/'+self.servicePath)
         self._role=role
-        self.network=net
+        if net!='defnetwork':
+            self.network=net
         self.destroy = self.DestroyTask(tasks=self)
         
         
     def initservice(self):
         super(LayerBuilder, self).__init__(self._service,
-                                      build_path='build/'+self.serviceName,
+                                      build_path='build/'+self.servicePath,
                                       roles=self._role,
                                       registry=self.registry,
                                       destroy_command=True)
@@ -422,26 +480,29 @@ class LayerConstructor(DockerTasks, Element):
     def __init__(self,
                 net='network',
                 role=['defrole'], 
-                dockerlogin=False, registry='',
+                dockerlogin=False,
+                #registry='',
                 projectname='testproj', 
                 imageTag='unitname' , 
                 release='latest'):
         self.dockerlogin=dockerlogin
         self.myname=projectname
-        self.registry=registry
+        #self.registry=registry
         self.imageTag=imageTag
-        self.serviceName=projectname+'/'+imageTag
+        self.serviceName=projectname+'-'+os.path.basename(imageTag)
+        self.servicePath=projectname+'/'+imageTag
         self.serviceTag=imageTag+':'+release
-        self.layerPath=os.path.join(os.getcwd(), self.serviceName)
+        self.layerPath=os.path.join(os.getcwd(), self.servicePath)
         self._role=role
-        self.network=net
+        if net!='defnetwork':
+            self.network=net
         self.destroy = self.DestroyTask(tasks=self)
     
         
     def initservice(self):
         super(LayerConstructor, self).__init__(self._service,
-                                               roles=self._role,
-                                               registry=self.registry)
+                                               roles=self._role)
+       #                                        registry=self.registry)
         self.dockerLogin()
              
     def hook(self):
@@ -457,7 +518,6 @@ class LayerFactorySkeel(ABC):
          
     def reset(self):
         self._element=None
-    def __increment(self):
         self.__index+=1
     
     @property
@@ -473,7 +533,9 @@ class LayerFactorySkeel(ABC):
         """
         self._element = element
 
-    
+    @abstractmethod
+    def updateService(self):
+        pass
 
 
    
