@@ -36,12 +36,15 @@ class ExecStrategy(ABC):
         self._element.initservice()
         self.accompaniment()
         self._element.updateFromImage(force=True, tag=tag)
+    
+    def setVolume(self, vol):
+        pass
          
     def makeVolume(self):
         if hasattr(self._element, '_Element__vols'):
             vols=self._element.volumes
             for vol in vols:
-                vols[vol].update()
+                vols[vol].update(self._element._role)
     
     def makeNetwork(self):
         if hasattr(self._element, '_Element__net'):
@@ -90,7 +93,7 @@ class CommonStrategy(ExecStrategy):
         
         
 class CommonStrategyForNginx(ExecStrategy):
-    
+    vols={}    
     def initRepo(self, branch='master'):
         repo=self._element.initRepoForLayer(branch)
         return repo
